;*************************************************************************
;*
;* Declare-Filter (c) 2007 - 2015 Markus Haacke
;* Tool fr die originale PureBasic IDE zur Erzeugung von Procedure-Deklarationen
;*
;*************************************************************************
;* 
;* Programname       : Declare-Filter
;* Filename          : Declare-Filter.pb
;* Filetype          : Main source   [Main source, Includefile, Data file]
;* Version           : 1.04
;* Date              : 30.04.2015
;* Autor             : Markus Haacke (www.gismeth.de)
;* Programming lang. : Purebasic 4.41+
;* Compile as        : Windows Exe
;* -----------------------------------------------------------------------
;* BESCHREIBUNG:
;* 
;* Das Programm wird als Werkzeug in die IDE des PureBasic Editors eingebunden. 
;* Es generiert fr die Proceduren des aktuellen Sourcecodes und den darin enthaltenen
;* Includes, die ntigen Deklarationen. Die generierten Declare-Anweisungen werden 
;* entweder in einer Fensteroberflche zum Kopieren in die Zwischenablage bereitgestellt
;* oder aber das Programm schreibt diese in eine seperate Datei, die wiederum vom 
;* ursprnglichen Sourcecode inkludiert werden kann.
;*
;* Der Dateiname des Hauptquellcodes kann als Kommandoparameter bergeben werden. In diesem
;* Fall arbeitet das Programm autonom ohne Nutzung der Fensteroberflche und schreibt
;* die Deklarationen in die o.g. separate Datei. Weiteres siehe Dokumentation.
;*
;* =======================================================================
;* *** GNU General Public License - informations in english ***
;*
;* This program is free software; you can redistribute it and/or modify it under the terms 
;* of the GNU General Public License Version 3 as published by the Free Software Foundation.
;*
;* This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY,
;* without even the implied warranty of MERCHANTABILITY or FITNESS for A PARTICULAR PURPOSE.
;* See the GNU General Public License for more details.
;*
;* You should have received a copy of the GNU General Public License along with this program.
;* If not, see <http://www.gnu.org/licenses/>.
;*
;* -----------------------------------------------------------------------
;* *** GNU General Public License - Informationen in Deutsch ***
;*
;* Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der GNU General 
;* Public License Version 3, wie von der Free Software Foundation verffentlicht, weitergeben
;* und/oder modifizieren.
;*
;* Die Verffentlichung dieses Programms erfolgt in der Hoffnung, da es Ihnen von Nutzen 
;* sein wird, aber OHNE IRGENDEINE GARANTIE, sogar ohne die implizite Garantie der MARKTREIFE
;* oder der VERWENDBARKEIT FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General 
;* Public License.
;* 
;* Sie sollten ein Exemplar der GNU General Public License zusammen mit diesem Programm 
;* erhalten haben. Falls nicht, siehe <http://www.gnu.org/licenses/>.
;*
;* =======================================================================
;* Historie:
;* 1.04 - 30.04.2015: Bugfix, Prozeduren mit Suffix (.s, .l usw.) wurden nicht korrekt erkannt.
;* 1.03 - 30.05.2010: Bugfix, Fr Prozeduren mit einem Array als Paremeter wurde eine falsche Deklaration erzeugt
;* 1.02 - 05.11.2007: Change, Erzeugung der Deklarationen-Datei erfolgt jetzt ohne API-Aufrufe (bessere Portierbarkeit auf andere Systeme)
;* 1.01 - 04.11.2007: Add, About-Dialog eingebaut und das Programm unter GNU GPL gestellt
;* 1.00 - 25.06.2007: Erste Version
;*
;*************************************************************************

EnableExplicit

#Version = "1.04 [30.04.2015]"
#Copyright = "2007 - 2015"

IncludeFile "GUI.pb"

Define Event.l, GadgetID.l ; ,WindowID.l, EventType.l
Global i.l
Global FileIn$, FileOut$, Parameters.b = #False, ValidFile.b = #False
Global NewList FileList.s()

; ----------------------------------------------------------------------------------

Procedure Editor_Save(Gadget.l, File.s)
	Protected Text$
	
	If CreateFile(0, File)
		Text$ = GetGadgetText(Gadget) 
		WriteData(0, @Text$, Len(Text$))
		CloseFile(0)
	  ProcedureReturn #True
	Else
	  ProcedureReturn #False
	EndIf
  
EndProcedure
Procedure CheckFile()

  If FileSize(FileIn$) > -1 ; Datei existiert
    SetGadgetText(#String_Datei, FileIn$)
    ValidFile = #True
  ElseIf FileIn$ <> ""
    MessageRequester("Fehler", "Ungltige Datei angegeben!" + Chr(10) + "<" + FileIn$ +">", #MB_ICONERROR)
    ValidFile = #False
  EndIf

EndProcedure
Procedure SaveFile(File$)

  If Editor_Save(#Editor_Liste, File$)
    ; Alles ist gut
  Else
    MessageRequester("Fehler", "Ausgabedatei konnte nicht geffnet werden!" + Chr(10) + "<" + FileOut$ +">", #MB_ICONERROR)
  EndIf
 
EndProcedure
Procedure FileRequester()

  FileIn$ = OpenFileRequester("Whle Datei...","","",1)

  If FileIn$ <> ""
    SetGadgetText(#String_Datei, FileIn$)
  Else 
    FileIn$ = GetGadgetText(#String_Datei)
  EndIf
  
  CheckFile()

EndProcedure

Procedure.i FindFileInList(File$)
	
  ResetList(FileList())
  While NextElement(FileList())
  	If FileList() = File$ : ProcedureReturn #True : EndIf
  Wend
  
  ProcedureReturn #False
		
EndProcedure
Procedure ScanForIncludes(File$)
	Protected iFile.i, IncFile$, Line$, Position.l, Length.l, CharsBefore.l, Declarefile.b = #False
	
	iFile = ReadFile(#PB_Any, File$)
	If iFile
		
    While (Not Eof(iFile))
      Line$ = ReadString(iFile)
  
      Repeat
        Position = 0
        If RTrim(LTrim(StringField(UCase(Line$), 1, Chr(34)))) = "INCLUDEFILE" ; Das erste Wort mu IncludeFile heissen
          Position = FindString(Line$, Chr(34), 1) ; Dann Suche nach dem ersten nachfolgenden Anfhrungszeichen

          If Position > 0
            Length = FindString(Line$, Chr(34), Position + 1) - Position ; Dann nach dem zweiten nachfolgenden Anfhrungszeichen

            If Length > 0
            	IncFile$ = GetPathPart(FileIn$) + Trim(ReplaceString(Mid(Line$, Position , Length), Chr(34), ""))
              Line$ = Right(Line$, Len(Line$) - (Position + Length))
              
              If Not FindFileInList(IncFile$)
	              AddElement(FileList())
	              FileList() = IncFile$ 
	              
	              If FindString(Line$, "%DECLAREFILE", 1) ; Falls das Magicword in der Zeile vorkommt, dann ist dies
	                FileOut$ = FileList()                  ; der Name des zu schreibenden Includefiles.
	                DeleteElement(FileList())
	              ElseIf FindString(Line$, "%IGNOREFILE", 1) ; Falls das Magicword in der Zeile vorkommt, dann wird
	                DeleteElement(FileList())                 ; dieses Include nicht bearbeitet.
	              Else
	              	ScanForIncludes(FileList())
	              	LastElement(FileList())
	              EndIf
	            EndIf
              
            EndIf
          EndIf
        EndIf
      Until Position = 0 Or Length = 0
  
    Wend
    CloseFile(iFile)
  EndIf
  
EndProcedure
Procedure ScanForProcedure(Filename$)
  Protected Line$, UpperLine$, LeftPosition.l, RightPosition.l, CommentPosition.l, ProcEndPosition.l, ProcRetPosition.l

  If ReadFile(0, Filename$)
    While (Not Eof(0))
      Line$ = ReadString(0)
      UpperLine$ = UCase(Line$)

      LeftPosition = FindString(UpperLine$, "PROCEDURE", 1)
      ; Es ist nur ein echtes "Procedure", wenn links neben dem "P" ein Leerzeichen oder ein ":" steht...
      If LeftPosition > 1
      	Select Asc(Mid(UpperLine$, LeftPosition	- 1, 1))
      		Case 32, 58
			      ; ... rechts neben dem "E" auch ein Leerzeichen, ein "." oder ein ":" steht...
			      Select Asc(Mid(UpperLine$, LeftPosition	+ 9, 1))
			      	Case 32, 46, 58
			      		;Alles ist gut
			      	Default
			      		LeftPosition = 0
			      		EndSelect
      		Default
      			LeftPosition = 0
      	EndSelect
      EndIf

;       ProcEndPosition = FindString(UpperLine$, "ENDPROCEDURE", 1)
;       If ProcEndPosition = 0 : ProcEndPosition = 9999 : EndIf
; 
;       ProcRetPosition = FindString(UpperLine$, "PROCEDURERETURN", 1)
;       If ProcRetPosition = 0 : ProcRetPosition = 9999 : EndIf

      CommentPosition = FindString(UpperLine$, ";", 1)
      If CommentPosition = 0 : CommentPosition = 9999 : EndIf
      
      If LeftPosition > 0 And LeftPosition < CommentPosition ; And LeftPosition < ProcEndPosition And LeftPosition < ProcRetPosition
      	LeftPosition + 9
      	
	      RightPosition = FindString(UpperLine$, ":", LeftPosition) 
				If RightPosition = 0
		      RightPosition = FindString(UpperLine$, ";", LeftPosition) 
				EndIf
	
				If RightPosition = 0
		      RightPosition = Len(Line$)
				EndIf
				
				Line$ = "Declare" + Mid(Line$, LeftPosition, RightPosition - LeftPosition + 1)
        AddGadgetItem(#Editor_Liste, -1, Line$)

      EndIf
         
    Wend
    CloseFile(0)

  Else
    AddGadgetItem(#Editor_Dateien, -1, "ACHTUNG: Datei " + Filename$ + " konnte nicht gelesen werden!")
  EndIf
  
EndProcedure
Procedure ExamineFiles()

  If ValidFile
    SetGadgetText(#Editor_Dateien, "") 
    SetGadgetText(#Editor_Liste, "") 

    ClearList(FileList())
    AddElement(FileList())
    FileList() = FileIn$

    ScanForIncludes(FileIn$)

    ForEach FileList()
      AddGadgetItem(#Editor_Dateien, -1, FileList())
    Next
  EndIf
  ProcedureReturn ValidFile

EndProcedure

Procedure Button_About()
	Protected About$
	
	Restore About
	Read.s About$
	About$ = ReplaceString(About$, "#COPYRIGHT", #Copyright)
	About$ = ReplaceString(About$, "#VERSION", #Version)
	
  MessageRequester("Informationen", About$, #MB_ICONINFORMATION)

EndProcedure
Procedure Button_Load()

  FileRequester()
  ExamineFiles()

EndProcedure
Procedure Button_Filter()

  If ExamineFiles()
    AddGadgetItem(#Editor_Liste, -1, "; --- Procedure declarations, generated by Declare-Filter " + #Version + " ---")
    ForEach FileList()
      AddGadgetItem(#Editor_Liste, -1, "")
      AddGadgetItem(#Editor_Liste, -1, "; --- " + GetFilePart(FileList()) )
      ScanForProcedure(FileList())
    Next
  EndIf

EndProcedure
Procedure Button_Kopieren()

  SetClipboardText(GetGadgetText(#Editor_Liste))

EndProcedure

Procedure CheckParameter()
  Protected NumPara.l, Filesize.l

  NumPara = CountProgramParameters()

  If NumPara > 0
    FileIn$ = ProgramParameter(0)
    CheckFile()

    If ValidFile
      Button_Filter()
    EndIf

    If NumPara > 1 ; Wenn zweiter Parameter, dann erstelle eine Includedatei mit den Declares...
      FileOut$ = ProgramParameter(1)
    EndIf
    Parameters = #True
  
  EndIf  

EndProcedure

; ----------------------------------------------------------------------------------

Open_Window_Main()
CheckParameter()

If FileOut$ <> "" And Parameters = #True

  SaveFile(FileOut$)

  OpenWindow(0, 10, 10, 370, 20, "Declare-Filter " + #Version + " (c) " + #Copyright + " Markus Haacke", #PB_Window_ScreenCentered) ; | #PB_Window_TitleBar)
 	TextGadget(0, 10, 3, 300, 20, "Deklarationen erstellt!")

  For i = 0 To 30 : Delay (10) : WindowEvent() : Next i
  CloseWindow(0)
  
  End
  
Else
  HideWindow(#Window_Main, 0)
EndIf

Repeat
  
  Event = WaitWindowEvent()
;  WindowID = EventWindow()
  GadgetID = EventGadget()
;  EventType = EventType()
  
  
  If Event = #PB_Event_Gadget
    
    If GadgetID = #ButtonImage_Load
      Button_Load()
    ElseIf GadgetID = #Button_About
      Button_About()
    ElseIf GadgetID = #Button_Filter
      Button_Filter()
    ElseIf GadgetID = #Button_Kopieren
      Button_Kopieren()
    EndIf

  EndIf
  
Until Event = #PB_Event_CloseWindow

End

DataSection
	About:	
	IncludeBinary "Data\Abouttext.txt" 
	Data.s " "
EndDataSection

